@@ -1,9 +1,8 @@
-/* $Xorg: cmsTrig.c,v 1.3 2000/08/17 19:45:10 cpqbld Exp $ */
 
 /*
  * Code and supporting documentation (c) Copyright 1990 1991 Tektronix, Inc.
  * 	All Rights Reserved
- * 
+ *
  * This file is a component of an X Window System-specific implementation
  * of Xcms based on the TekColor Color Management System.  Permission is
  * hereby granted to use, copy, modify, sell, and otherwise distribute this
@@ -11,10 +10,10 @@
  * that this copyright, permission, and disclaimer notice is reproduced in
  * all copies of this software and in supporting documentation.  TekColor
  * is a trademark of Tektronix, Inc.
- * 
+ *
  * Tektronix makes no representation about the suitability of this software
  * for any purpose.  It is provided "as is" and with all faults.
- * 
+ *
  * TEKTRONIX DISCLAIMS ALL WARRANTIES APPLICABLE TO THIS SOFTWARE,
  * INCLUDING THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  * PARTICULAR PURPOSE.  IN NO EVENT SHALL TEKTRONIX BE LIABLE FOR ANY
@@ -24,7 +23,6 @@
  * CONNECTION WITH THE USE OR THE PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $XFree86: xc/lib/X11/cmsTrig.c,v 3.8 2001/10/28 03:32:34 tsi Exp $ */
 /*
  *	It should be pointed out that for simplicity's sake, the
  *	environment parameters are defined as floating point constants,
@@ -82,12 +80,8 @@ _XcmsModuloF(
 #define XCMS_FABS(x)		((x) < 0.0 ? -(x) : (x))
 
 /* XCMS_DMAXPOWTWO - largest power of two exactly representable as a double */
-#ifdef _CRAY
-#define XCMS_DMAXPOWTWO	((double)(1 < 47))
-#else
 #define XCMS_DMAXPOWTWO	((double)(XCMS_LONG_MAX) * \
 	    (1L << ((XCMS_NBITS(double)-XCMS_DEXPLEN) - XCMS_NBITS(int) + 1)))
-#endif
 
 /*
  *	LOCAL VARIABLES
@@ -165,7 +159,7 @@ static double const sin_qcoeffs[] = {
  *	to the range over which the polynomial approximation
  *	is valid, and as the polynomial is evaluated using
  *	finite-precision arithmetic.
- *	
+ *
  *  PROGRAMMER
  *
  *	Fred Fish
@@ -225,7 +219,7 @@ static double const sin_qcoeffs[] = {
  *	HART satisfies this requirement, although not explicitly
  *	stated in the text.  This may not be true of other
  *	approximations given in the reference.
- *			
+ *
  */
 
 double _XcmsCosine(double x)
@@ -444,7 +438,7 @@ static double _XcmsPolynomial(
  *	to the range over which the polynomial approximation
  *	is valid, and as the polynomial is evaluated using
  *	finite-precision arithmetic.
- *	
+ *
  *  PROGRAMMER
  *
  *	Fred Fish
@@ -505,7 +499,7 @@ static double _XcmsPolynomial(
  *	HART satisfies this requirement, although not explicitly
  *	stated in the text.  This may not be true of other
  *	approximations given in the reference.
- *			
+ *
  */
 
 double
@@ -559,7 +553,7 @@ _XcmsArcTangent(double x)
  *			New York, NY, Harper & Row, 1970.
  *
  *	RETURNS
- *		Returns the arctangent 
+ *		Returns the arctangent
  */
 {
     double ai, a1 = 0.0, bi, b1 = 0.0, l, d;
